# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/sales/picking_order.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1cv1/sales/picking_order.proto\x12\'pro.omni.oms.api.v1.sales.picking.order\x1a\x11\x63ommon/base.proto"\x91\x01\n\x0cPickingOrder\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x10\n\x08order_id\x18\x02 \x01(\x05\x12\x12\n\npicking_id\x18\x03 \x01(\x05\x12\x0e\n\x06\x61\x63tive\x18\x04 \x01(\x08\x12?\n\x0cobject_audit\x18\x05 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"y\n\x19PickingOrderCreateRequest\x12\x10\n\x08order_id\x18\x01 \x01(\x05\x12\x12\n\npicking_id\x18\x02 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xb5\x01\n\x1aPickingOrderCreateResponse\x12L\n\rpicking_order\x18\x01 \x01(\x0b\x32\x35.pro.omni.oms.api.v1.sales.picking.order.PickingOrder\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xf5\x02\n\x17PickingOrderReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xef\x01\n\x18PickingOrderReadResponse\x12M\n\x0epicking_orders\x18\x01 \x03(\x0b\x32\x35.pro.omni.oms.api.v1.sales.picking.order.PickingOrder\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x03 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData"\xa1\x01\n\x19PickingOrderUpdateRequest\x12L\n\rpicking_order\x18\x01 \x01(\x0b\x32\x35.pro.omni.oms.api.v1.sales.picking.order.PickingOrder\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xb5\x01\n\x1aPickingOrderUpdateResponse\x12L\n\rpicking_order\x18\x01 \x01(\x0b\x32\x35.pro.omni.oms.api.v1.sales.picking.order.PickingOrder\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"_\n\x19PickingOrderDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xb5\x01\n\x1aPickingOrderDeleteResponse\x12L\n\rpicking_order\x18\x01 \x01(\x0b\x32\x35.pro.omni.oms.api.v1.sales.picking.order.PickingOrder\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\x97\x05\n\x13PickingOrderService\x12\x9f\x01\n\x12PickingOrderCreate\x12\x42.pro.omni.oms.api.v1.sales.picking.order.PickingOrderCreateRequest\x1a\x43.pro.omni.oms.api.v1.sales.picking.order.PickingOrderCreateResponse"\x00\x12\x99\x01\n\x10PickingOrderRead\x12@.pro.omni.oms.api.v1.sales.picking.order.PickingOrderReadRequest\x1a\x41.pro.omni.oms.api.v1.sales.picking.order.PickingOrderReadResponse"\x00\x12\x9f\x01\n\x12PickingOrderUpdate\x12\x42.pro.omni.oms.api.v1.sales.picking.order.PickingOrderUpdateRequest\x1a\x43.pro.omni.oms.api.v1.sales.picking.order.PickingOrderUpdateResponse"\x00\x12\x9f\x01\n\x12PickingOrderDelete\x12\x42.pro.omni.oms.api.v1.sales.picking.order.PickingOrderDeleteRequest\x1a\x43.pro.omni.oms.api.v1.sales.picking.order.PickingOrderDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.sales.picking_order_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_PICKINGORDER"]._serialized_start = 93
    _globals["_PICKINGORDER"]._serialized_end = 238
    _globals["_PICKINGORDERCREATEREQUEST"]._serialized_start = 240
    _globals["_PICKINGORDERCREATEREQUEST"]._serialized_end = 361
    _globals["_PICKINGORDERCREATERESPONSE"]._serialized_start = 364
    _globals["_PICKINGORDERCREATERESPONSE"]._serialized_end = 545
    _globals["_PICKINGORDERREADREQUEST"]._serialized_start = 548
    _globals["_PICKINGORDERREADREQUEST"]._serialized_end = 921
    _globals["_PICKINGORDERREADRESPONSE"]._serialized_start = 924
    _globals["_PICKINGORDERREADRESPONSE"]._serialized_end = 1163
    _globals["_PICKINGORDERUPDATEREQUEST"]._serialized_start = 1166
    _globals["_PICKINGORDERUPDATEREQUEST"]._serialized_end = 1327
    _globals["_PICKINGORDERUPDATERESPONSE"]._serialized_start = 1330
    _globals["_PICKINGORDERUPDATERESPONSE"]._serialized_end = 1511
    _globals["_PICKINGORDERDELETEREQUEST"]._serialized_start = 1513
    _globals["_PICKINGORDERDELETEREQUEST"]._serialized_end = 1608
    _globals["_PICKINGORDERDELETERESPONSE"]._serialized_start = 1611
    _globals["_PICKINGORDERDELETERESPONSE"]._serialized_end = 1792
    _globals["_PICKINGORDERSERVICE"]._serialized_start = 1795
    _globals["_PICKINGORDERSERVICE"]._serialized_end = 2458
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import (
    delivery_method_warehouse_hierarchy_pb2 as v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2,
)


class DeliveryMethodWarehouseHierarchyServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.DeliveryMethodWarehouseHierarchyCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyCreate",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyCreateResponse.FromString,
        )
        self.DeliveryMethodWarehouseHierarchyRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyRead",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyReadResponse.FromString,
        )
        self.DeliveryMethodWarehouseHierarchyUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyUpdate",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyUpdateResponse.FromString,
        )
        self.DeliveryMethodWarehouseHierarchyDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyDelete",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyDeleteResponse.FromString,
        )


class DeliveryMethodWarehouseHierarchyServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def DeliveryMethodWarehouseHierarchyCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeliveryMethodWarehouseHierarchyRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeliveryMethodWarehouseHierarchyUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeliveryMethodWarehouseHierarchyDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_DeliveryMethodWarehouseHierarchyServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "DeliveryMethodWarehouseHierarchyCreate": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseHierarchyCreate,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyCreateResponse.SerializeToString,
        ),
        "DeliveryMethodWarehouseHierarchyRead": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseHierarchyRead,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyReadResponse.SerializeToString,
        ),
        "DeliveryMethodWarehouseHierarchyUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseHierarchyUpdate,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyUpdateResponse.SerializeToString,
        ),
        "DeliveryMethodWarehouseHierarchyDelete": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseHierarchyDelete,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService",
        rpc_method_handlers,
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class DeliveryMethodWarehouseHierarchyService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def DeliveryMethodWarehouseHierarchyCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyCreate",
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyCreateRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeliveryMethodWarehouseHierarchyRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyRead",
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyReadRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeliveryMethodWarehouseHierarchyUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyUpdate",
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyUpdateRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeliveryMethodWarehouseHierarchyDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyService/DeliveryMethodWarehouseHierarchyDelete",
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyDeleteRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__hierarchy__pb2.DeliveryMethodWarehouseHierarchyDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



import ast
import json
import operator
import time

import mongoengine as mongo
import redis
from bson import ObjectId
from omni.pro.logger import configure_logger
from omni.pro.protos.common import base_pb2
from omni.pro.util import nested
from peewee import Expression, Model, ModelSelect, PostgresqlDatabase

logger = configure_logger(name=__name__)


def measure_time(function):
    def measured_function(*args, **kwargs):
        start = time.time()
        c = function(*args, **kwargs)
        # logger.info(f"Func: {str(function.__qualname__)} - Time: {time.time() - start}")
        return c

    return measured_function


class DatabaseManager(object):
    def __init__(self, host: str, port: int, db: str, user: str, password: str, complement: dict) -> None:
        """
        :param db_object: Database object
        Example:
            db_object = {
                "host":"mongo",
                "port":"27017",
                "user":"root",
                "password":"123456",
                "type":"write | read",
                "no_sql":"true",
                "complement":""
            }
        """
        self.db = db
        self.host = host
        self.port = port
        self.username = user
        self.password = password
        self.complement = complement
        # self.get_connection().connect()

    def get_connection(self):
        return MongoConnection(
            db=self.db,
            host=self.host,
            port=self.port,
            username=self.username,
            password=self.password,
            complement=self.complement,
        )

    def create_document(self, db_name: str, document_class, **kwargs) -> object:
        document = document_class(**kwargs)
        document.save()
        return document

    def get_document(self, db_name: str, tenant: str, document_class, **kwargs) -> object:
        document = document_class.objects(**kwargs, context__tenant=tenant).first()
        # document.to_proto()
        return document

    def update_document(self, db_name: str, document_class, id: str, **kwargs) -> object:
        document = document_class.objects(id=id).first()
        document_class.objects(id=document.id).update_one(**kwargs)
        document.reload()
        return document

    def update(self, document_instance, **kwargs):
        document_instance.update(**kwargs)
        document_instance.reload()
        return document_instance

    def delete(self, document_instance):
        document_instance.delete()
        return document_instance

    def delete_document(self, db_name: str, document_class, id: str) -> object:
        document = document_class.objects(id=id).first()
        document.delete()
        return document

    def list_documents(
        self,
        db_name: str,
        tenant: str,
        document_class,
        fields: list = None,
        filter: dict = None,
        group_by: str = None,
        paginated: dict = None,
        sort_by: list = None,
    ) -> tuple[list, int]:
        """
        Parameters:
        fields (list): Optional list of fields to retrieve from the documents.
        filter (dict): Optional dictionary containing filter criteria for the query.
        group_by (str): Optional field to group results by.
        paginated (dict): Optional dictionary containing pagination information.
        sort_by (list): Optional list of fields to sort results by.

        Returns:
        list: A list of documents matching the specified criteria.
        """
        # Filter documents based on criteria provided

        if filter:
            query_set = document_class.objects(context__tenant=tenant).filter(__raw__=filter)
        else:
            query_set = document_class.objects(context__tenant=tenant)

        # Only retrieve specified fields
        if fields:
            query_set = query_set.only(*fields)

        # Group results by specified field
        if group_by:
            query_set = query_set.group_by(group_by)

        # Paginate results based on criteria provided
        if paginated:
            page = int(paginated.get("page") or 1)
            per_page = int(paginated.get("per_page") or 10)
            start = (page - 1) * per_page
            end = start + per_page
            query_set = query_set[start:end]

        # Sort results based on criteria provided
        if sort_by:
            query_set = query_set.order_by(*sort_by)

        # Return list of documents matching the specified criteria and total count of documents
        return list(query_set), query_set.count()

    def delete_documents(self, db_name, document_class, **kwargs):
        # with self.get_connection() as cnn:
        document = document_class.objects(**kwargs).delete()
        return document

    def update_embeded_document(
        self, db_name: str, document_class, filters: dict, update: dict, many: bool = False
    ) -> object:
        # with self.get_connection() as cnn:
        if many:
            document_class.objects(**filters).update(**update)
            document = document_class.objects(**filters)
        else:
            document_class.objects(**filters).update_one(**update)
            document = document_class.objects(**filters).first()
        return document


class MongoConnection(object):
    """A MongoConnection class that can dynamically connect to a MongoDB database with MongoEngine and close the connection after each query.

    Args:
        host (str): The hostname or IP address of the MongoDB server.
        port (int): The port number of the MongoDB server.
        username (str): The username for the MongoDB database.
        password (str): The password for the MongoDB database.
        database (str): The name of the MongoDB database.
    """

    def __init__(self, host, port, db, username, password, complement):
        self.host = f"mongodb://{host}:{port}/?{'&'.join([f'{k}={v}' for (k,v) in complement.items()])}"
        self.port = port
        self.username = username
        self.password = password
        self.db = db

    def connect(self):
        """Connects to the MongoDB database.

        Returns:
            A MongoEngine connection object.
        """
        self.connection = mongo.connect(
            db=self.db,
            username=self.username,
            password=self.password,
            host=self.host,
        )
        return self.connection

    def close(self):
        """Closes the connection to the MongoDB database."""
        # self.connection.close()
        mongo.disconnect()

    def __enter__(self):
        """Enters a context manager.

        Returns:
            A MongoConnection object.
        """
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Exits a context manager."""
        self.close()


class PostgresDatabaseManager:
    def __init__(self, name: str, host: str, port: str, user: str, password: str):
        self.name = name
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.connection = PostgresqlDatabase(
            database=self.name,
            user=self.user,
            password=self.password,
            host=self.host,
            port=self.port,
        )

    def get_db_connection(self):
        return self.connection

    def create_table_if_not_exists(self, model: Model):
        model._meta.database = self.connection
        with self.connection.atomic():
            if model.table_exists():
                return
            self.connection.create_tables([model])

    def create_new_record(self, model: Model, **kwargs):
        model._meta.database = self.connection
        with self.connection.atomic():
            self.create_table_if_not_exists(model)
            new_record = model(**kwargs)
            new_record.bind(self.connection)
            new_record.save()
        return new_record

    def retrieve_record(self, model: Model, filters: dict):
        model._meta.database = self.connection
        with self.connection.atomic():
            model.bind(self.connection)
            self.create_table_if_not_exists(model)
            query = model.get_or_none(**filters)
        return query

    def retrieve_record_by_id(self, model: Model, id: int):
        model._meta.database = self.connection
        with self.connection.atomic():
            model.bind(self.connection)
            self.create_table_if_not_exists(model)
            query = model.get_by_id(id)
        return query

    def list_records(
        self,
        model: Model,
        id: int,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
        paginated: base_pb2.Paginated,
    ):
        with self.connection.atomic():
            model.bind(self.connection)
            self.create_table_if_not_exists(model)
            model_select = QueryBuilder().build_filter(model, id, fields, filter, group_by, sort_by, paginated)
        return model_select

    def update_record(self, model, model_id, update_dict):
        model._meta.database = self.connection
        with self.connection.atomic():
            model.bind(self.connection)
            self.create_table_if_not_exists(model)
            record = model.get_by_id(model_id)
            record.update(**update_dict).where(model.id == model_id).execute()
        return model.get_by_id(model_id)

    def delete_record_by_id(self, model, model_id):
        model._meta.database = self.connection
        with self.connection.atomic():
            model.bind(self.connection)
            self.create_table_if_not_exists(model)
            record = model.get_by_id(model_id)
            query = record.delete_instance()
        return query


class RedisConnection:
    def __init__(self, host: str, port: int, db: int) -> None:
        self.host = host
        self.port = int(port)
        self.db = db

    def __enter__(self) -> redis.StrictRedis:
        self.redis_client = redis.StrictRedis(host=self.host, port=self.port, db=self.db, decode_responses=True)
        return self.redis_client

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.redis_client.close()


class RedisManager(object):
    def __init__(self, host: str, port: int, db: int) -> None:
        self.host = host
        self.port = int(port)
        self.db = db
        self._connection = RedisConnection(host=self.host, port=self.port, db=self.db)

    def get_connection(self) -> RedisConnection:
        return self._connection

    def set_connection(self, connection: RedisConnection) -> None:
        self._connection = connection

    def set_json(self, key, json_obj):
        with self.get_connection() as rc:
            if isinstance(json_obj, str):
                json_obj = json.loads(json_obj)
            return rc.json().set(key, "$", json_obj)

    def get_json(self, key):
        with self.get_connection() as rc:
            return rc.json().get(key)

    def get_resource_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_json(tenant_code)
        # logger.info(f"Redis config", extra={"deb_config": config})
        return {
            **nested(config, f"resources.{service_id}", {}),
            **nested(config, "aws", {}),
        }

    def get_aws_cognito_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "region_name": nested(config, "aws.cognito.region"),
            "aws_access_key_id": config.get("aws_access_key_id"),
            "aws_secret_access_key": config.get("aws_secret_access_key"),
            "user_pool_id": nested(config, "aws.cognito.user_pool_id"),
        }

    def get_mongodb_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "host": nested(config, "dbs.mongodb.host"),
            "port": nested(config, "dbs.mongodb.port"),
            "user": nested(config, "dbs.mongodb.user"),
            "password": nested(config, "dbs.mongodb.pass"),
            "name": nested(config, "dbs.mongodb.name"),
            "complement": nested(config, "dbs.mongodb.complement"),
        }

    def get_postgres_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "host": nested(config, "dbs.postgres.host"),
            "port": nested(config, "dbs.postgres.port"),
            "user": nested(config, "dbs.postgres.user"),
            "password": nested(config, "dbs.postgres.pass"),
            "name": nested(config, "dbs.postgres.name"),
        }

    def get_tenant_codes(self, pattern="*") -> list:
        with self.get_connection() as rc:
            return rc.keys(pattern=pattern)


class PolishNotationToMongoDB:
    def __init__(self, expression):
        self.expression = expression
        self.operators_logical = {"and": "$and", "or": "$or", "nor": "$nor", "not": "$not"}
        self.operators_comparison = {
            "=": "$eq",
            ">": "$gt",
            "<": "$lt",
            ">=": "$gte",
            "<=": "$lte",
            "in": "$in",
            "nin": "$nin",
            "!=": "$ne",
        }

    def is_logical_operator(self, token):
        if not isinstance(token, str):
            return False
        return token in self.operators_logical

    def is_comparison_operator(self, token):
        if not isinstance(token, str):
            return False
        return token in self.operators_comparison

    def is_tuple(self, token):
        return isinstance(token, tuple) and len(token) == 3

    def convert(self):
        operand_stack = []
        operator_stack = []

        for token in reversed(self.expression):
            if self.is_logical_operator(token):
                operator_stack.append(token)
            elif self.is_comparison_operator(token):
                operator_stack.append(token)
            elif self.is_tuple(token):
                field, old_operator, value = token
                if old_operator in self.operators_comparison:
                    operand_stack.append({field: {self.operators_comparison[old_operator]: value}})
                else:
                    raise ValueError(f"Unexpected operator: {old_operator}")
            else:
                raise ValueError(f"Unexpected token: {token}")

        while operator_stack:
            operator = operator_stack.pop()
            if operator in self.operators_logical:
                operands = []
                for _ in range(2):
                    operands.append(operand_stack.pop())
                operand_stack.append({self.operators_logical[operator]: operands})
            else:
                raise ValueError(f"Unexpected operator: {operator}")

        return operand_stack.pop()


class DBUtil(object):
    @classmethod
    def db_prepared_statement(
        cls,
        id: str,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        paginated: base_pb2.Paginated,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
    ) -> dict:
        prepared_statement = {}
        prepared_statement["paginated"] = {"page": paginated.offset, "per_page": paginated.limit or 10}
        if (ft := filter.ListFields()) or id:
            expression = [("_id", "=", cls.generate_object_id(id))]
            if ft:
                str_filter = filter.filter.replace("true", "True").replace("false", "False")
                expression = ast.literal_eval(str_filter)
            filter_custom = PolishNotationToMongoDB(expression=expression).convert()
            prepared_statement["filter"] = filter_custom
        if group_by:
            prepared_statement["group_by"] = [x.name_field for x in group_by]
        if sort_by.ListFields():
            prepared_statement["sort_by"] = [cls.db_trans_sort(sort_by)]
        return prepared_statement

    @classmethod
    def db_trans_sort(cls, sort_by: base_pb2.SortBy) -> str:
        if not sort_by.name_field:
            return None
        return f"{'-' if sort_by.type == sort_by.DESC else '+'}{sort_by.name_field}"

    @classmethod
    def generate_object_id(cls, id=None):
        try:
            return ObjectId(id)
        except:
            return ObjectId(None)


class QueryBuilder:
    filter_ops = {
        "=": operator.eq,
        "<": operator.lt,
        "<=": operator.le,
        ">": operator.gt,
        ">=": operator.ge,
        "!=": operator.ne,
        "in": operator.contains,
        "and": operator.and_,
        "or": operator.or_,
        "not": lambda x: ~(x),
    }
    DEFAULT_PAGE_SIZE = 10

    @classmethod
    def pre_to_in(cls, filters: base_pb2.Filter) -> list:
        str_filter = filters.filter.replace("true", "True").replace("false", "False")
        expression = ast.literal_eval(str_filter)
        stack = []
        result = []
        for item in expression:
            if isinstance(item, tuple):
                if stack:
                    result.append(item)
                    result.append(stack.pop())
                else:
                    result.append(item)
            else:
                stack.append(item)
        return result

    @classmethod
    def build_filter(
        cls,
        model: Model,
        id: int,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
        paginated: base_pb2.Paginated,
    ):
        query_fields = list()
        query: Expression | None = None
        group_by_fields = list()
        order_by_fields = list()
        model_select: ModelSelect
        total: int = 0

        if id:
            model_select = model.select().where(model.id == id)
            return list(model_select), model_select.count()

        if fields.ListFields():
            query_fields = cls.build_fields(model, fields)

        if filter.ListFields():
            filter_custom = cls.pre_to_in(filter)
            query = cls.build_query(model, filter_custom)

        total = model.select(*query_fields).where(query).count()

        if paginated.ListFields():
            model_select = (
                model.select(*query_fields)
                .where(query)
                .paginate(paginated.offset, paginated.limit or cls.DEFAULT_PAGE_SIZE)
            )
        else:
            model_select = model.select(*query_fields).where(query).paginate(1, cls.DEFAULT_PAGE_SIZE)

        if group_by:
            group_by_fields = cls.build_group_by(model, group_by)
            model_select = model_select.group_by(*group_by_fields)

        if sort_by.ListFields():
            order_by_fields = cls.build_sort_by(model, sort_by)
            model_select = model_select.order_by(*order_by_fields)

        return list(model_select), total

    @classmethod
    def build_query(cls, model: Model, filters: list) -> Expression | None:
        query: Expression | None = None
        filter_operator = None
        for item in filters:
            if isinstance(item, str):
                filter_operator = item.lower()
                if filter_operator not in cls.filter_ops:
                    raise ValueError(f"Invalid filter operator: {filter_operator}")
            elif isinstance(item, tuple) and len(item) == 3:
                field, op, value = item
                # TODO: add support for related fields
                if "." in field:
                    related, field = field.split(".")
                    related_model = getattr(model, related).rel_model
                    related_field = getattr(related_model, field)
                    related_query = cls.filter_ops[op](related_field, value)
                    query = cls.filter_ops[op](query, related_query) if query else related_query
                else:
                    field = getattr(model, field)
                    field_query = cls.filter_ops[op](field, value)
                    query = cls.filter_ops[filter_operator](query, field_query) if filter_operator else field_query
            else:
                raise ValueError(f"Invalid filter item: {item}")

        return query

    @classmethod
    def build_fields(cls, model: Model, fields: base_pb2.Fields) -> list:
        return [getattr(model, x) for x in fields.name_field]

    @classmethod
    def build_group_by(cls, model: Model, group_by: base_pb2.GroupBy) -> list:
        group_by_fields = list()
        for item in group_by:
            if item.name_field:
                group_by_fields.append(getattr(model, item.name_field))
        return group_by_fields

    @classmethod
    def build_group_by(cls, model: Model, group_by: base_pb2.GroupBy) -> list:
        return [getattr(model, x.name_field) for x in group_by]

    @classmethod
    def build_sort_by(cls, model: Model, sort_by: base_pb2.SortBy) -> list:
        if sort_by.type == sort_by.DESC:
            return [getattr(model, sort_by.name_field).desc()]
        return [getattr(model, sort_by.name_field)]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/delivery_method_warehouse_hierarchy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n2v1/rules/delivery_method_warehouse_hierarchy.proto\x12=pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy\x1a\x11\x63ommon/base.proto"\xc5\x01\n DeliveryMethodWarehouseHierarchy\x12\n\n\x02id\x18\x01 \x01(\x05\x12$\n\x1c\x64\x65livery_method_warehouse_id\x18\x02 \x01(\x05\x12\x1e\n\x16warehouse_hierarchy_id\x18\x03 \x01(\x05\x12\x0e\n\x06\x61\x63tive\x18\x04 \x01(\x08\x12?\n\x0cobject_audit\x18\x05 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xad\x01\n-DeliveryMethodWarehouseHierarchyCreateRequest\x12$\n\x1c\x64\x65livery_method_warehouse_id\x18\x01 \x01(\x05\x12\x1e\n\x16warehouse_hierarchy_id\x18\x02 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8a\x02\n.DeliveryMethodWarehouseHierarchyCreateResponse\x12\x8c\x01\n#delivery_method_warehouse_hierarchy\x18\x01 \x01(\x0b\x32_.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchy\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x89\x03\n+DeliveryMethodWarehouseHierarchyReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xc4\x02\n,DeliveryMethodWarehouseHierarchyReadResponse\x12\x8d\x01\n$delivery_method_warehouse_hierarchys\x18\x01 \x03(\x0b\x32_.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchy\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xf6\x01\n-DeliveryMethodWarehouseHierarchyUpdateRequest\x12\x8c\x01\n#delivery_method_warehouse_hierarchy\x18\x01 \x01(\x0b\x32_.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchy\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8a\x02\n.DeliveryMethodWarehouseHierarchyUpdateResponse\x12\x8c\x01\n#delivery_method_warehouse_hierarchy\x18\x01 \x01(\x0b\x32_.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchy\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"s\n-DeliveryMethodWarehouseHierarchyDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"{\n.DeliveryMethodWarehouseHierarchyDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xcb\x08\n\'DeliveryMethodWarehouseHierarchyService\x12\x87\x02\n&DeliveryMethodWarehouseHierarchyCreate\x12l.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyCreateRequest\x1am.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyCreateResponse"\x00\x12\x81\x02\n$DeliveryMethodWarehouseHierarchyRead\x12j.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyReadRequest\x1ak.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyReadResponse"\x00\x12\x87\x02\n&DeliveryMethodWarehouseHierarchyUpdate\x12l.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyUpdateRequest\x1am.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyUpdateResponse"\x00\x12\x87\x02\n&DeliveryMethodWarehouseHierarchyDelete\x12l.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyDeleteRequest\x1am.pro.omni.oms.api.v1.rules.delivery_method_warehouse_hierarchy.DeliveryMethodWarehouseHierarchyDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.delivery_method_warehouse_hierarchy_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHY"]._serialized_start = 137
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHY"]._serialized_end = 334
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYCREATEREQUEST"]._serialized_start = 337
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYCREATEREQUEST"]._serialized_end = 510
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYCREATERESPONSE"]._serialized_start = 513
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYCREATERESPONSE"]._serialized_end = 779
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYREADREQUEST"]._serialized_start = 782
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYREADREQUEST"]._serialized_end = 1175
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYREADRESPONSE"]._serialized_start = 1178
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYREADRESPONSE"]._serialized_end = 1502
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYUPDATEREQUEST"]._serialized_start = 1505
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYUPDATEREQUEST"]._serialized_end = 1751
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYUPDATERESPONSE"]._serialized_start = 1754
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYUPDATERESPONSE"]._serialized_end = 2020
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYDELETEREQUEST"]._serialized_start = 2022
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYDELETEREQUEST"]._serialized_end = 2137
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYDELETERESPONSE"]._serialized_start = 2139
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYDELETERESPONSE"]._serialized_end = 2262
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYSERVICE"]._serialized_start = 2265
    _globals["_DELIVERYMETHODWAREHOUSEHIERARCHYSERVICE"]._serialized_end = 3364
# @@protoc_insertion_point(module_scope)



from omni.pro.models.base import BaseModel
from omni.pro.models.sql.rules.schedule_work import ScheduleWork
from omni.pro.models.sql.rules.schedule_work_line import ScheduleWorkLine
from omni.pro.protos.v1.rules.schedule_work_schedule_work_line_pb2 import (
    ScheduleWorkScheduleWorkLine as ScheduleWorkScheduleWorkLineProto,
)
from peewee import DeferredThroughModel, ForeignKeyField


# TODO tabla intermedia entre schedule_work y schedule_work_line
class ScheduleWorkScheduleWorkLine(BaseModel):
    schedule_work_id = ForeignKeyField(ScheduleWork, on_delete="RESTRICT")
    schedule_work_line_id = ForeignKeyField(ScheduleWorkLine, on_delete="RESTRICT")

    def to_proto(self):
        return ScheduleWorkScheduleWorkLineProto(
            id=self.id,
            schedule_work_id=self.schedule_work_id.id,
            schedule_work_line_id=self.schedule_work_line_id.id,
            active=self.active,
            object_audit=self.get_audit_proto(),
        )

    class Meta:
        table_name = "schedule_work_schedule_work_line"


DeferredThroughModel.set_model(ScheduleWork.schedule_work_lines_ids, ScheduleWorkScheduleWorkLine)



from mongoengine import fields
from omni.pro.models.base import BaseDocument


class MicroService(BaseDocument):
    name = fields.StringField()
    code = fields.StringField()
    tenant_code = fields.StringField()
    summary = fields.StringField()
    description = fields.StringField()
    author = fields.StringField()
    category = fields.StringField()
    version = fields.StringField()
    depends = fields.ListField(fields.StringField())
    data = fields.ListField(fields.DictField())



import csv
import importlib
from pathlib import Path

import grpc
from google.protobuf import wrappers_pb2
from omni.pro import redis, util
from omni.pro.cloudmap import CloudMap
from omni.pro.config import Config
from omni.pro.database import DatabaseManager
from omni.pro.logger import LoggerTraceback, configure_logger
from omni.pro.models.common.ms import MicroService
from omni.pro.protos.v1.users import user_pb2, user_pb2_grpc
from omni.pro.validators import MicroServiceValidator

logger = configure_logger(name=__name__)


class LoadData(object):
    def __init__(self, base_app: Path):
        # self.redis_manager = redis.get_redis_manager()
        self.base_app = base_app
        self.manifest = self._get_manifest_ms()
        # self.context = self._get_context()
        self.microserivce = None

    def load_data(self, *args, **kwargs):
        self.redis_manager = redis.get_redis_manager()
        list_contexts = []
        with self.redis_manager.get_connection() as rc:
            tenant_codes = self.redis_manager.get_tenant_codes()
            for tenant in tenant_codes:
                user = rc.json().get(tenant, "$.user_admin")
                context = {"tenant": tenant, "user": user}
                if (not context.get("user")) or (context.get("user") and (not context.get("user")[0])):
                    context["user"] = None
                    response, success = self.create_user_admin(context, rc)
                    if not success:
                        # TODO: End process
                        continue
                    context["user"] = response.user.username
                else:
                    # TODO: Validate if user exists in database
                    context["user"] = user[0].get("username")
                db_params = self.redis_manager.get_mongodb_config(Config.SERVICE_ID, tenant)
                db_params["db"] = db_params.pop("name")
                self.db_manager = DatabaseManager(**db_params)
                with self.db_manager.get_connection():
                    if not self.manifest:
                        continue
                    micro = self.load_manifest(context=context)
                    self.load_data_micro(micro, context)
                list_contexts.append(context)
        return list_contexts

    def create_user_admin(self, context, rc):
        values = self.load_data_dict(Path(__file__).parent / "data" / "models.user.csv")
        user = UserStub()
        response = user.create_users(context, values)
        success = response.response_standard.status_code in range(200, 300)
        if success:
            rc.json().set(
                context.get("tenant"), "$.user_admin", {"id": response.user.id, "username": response.user.username}
            )
        return response, success

    def _get_manifest_ms(self):
        file_name = self.base_app / "__manifest__.py"
        if not file_name.exists():
            logger.warning(f"Manifest file not found {file_name}")
            return {}
        with open(file_name, "r") as f:
            data = f.read()
        return eval(data)

    def load_manifest(self, context: dict):
        tenant = context.get("tenant")
        filters = {"code": self.manifest.get("code"), "tenant_code": tenant}
        micro: MicroService = self.db_manager.get_document(
            db_name=None, tenant=tenant, document_class=MicroService, **filters
        )
        self.manifest["tenant_code"] = tenant
        data_validated = MicroServiceValidator(self.base_app, micro.data if micro else []).load(
            self.manifest | {"context": context}
        )
        if not micro:
            micro = self.db_manager.create_document(db_name=None, document_class=MicroService, **data_validated)
        else:
            micro.data = data_validated.get("data")
            micro.save()
        self.microservice = micro
        return micro

    def load_data_dict(self, name_file):
        try:
            with open(name_file, mode="r", encoding="utf-8-sig") as csv_file:
                reader = csv.DictReader(csv_file, delimiter=";")
                for row in reader:
                    yield row
                return reader
        except FileNotFoundError as e:
            LoggerTraceback.error("File not found exception", e, logger)
        except Exception as e:
            LoggerTraceback.error("An unexpected error has occurred", e, logger)

    def load_data_micro(self, micro: MicroService, context: dict):
        tenant = context.get("tenant")
        for idx, file in enumerate(micro.data):
            if file.get("load"):
                continue
            name_file = self.base_app / file.get("path")
            reader = self.load_data_dict(name_file)
            models, file_py, model_str = file.get("path").split("/")[1].split(".")[:-1]
            model_str = util.to_camel_case(model_str)
            ruta_modulo = self.base_app / models / f"{file_py}.py"
            if not ruta_modulo.exists():
                logger.error(f"File not found {ruta_modulo}")
                continue
            modulo = util.load_file_module(ruta_modulo, model_str)
            if not hasattr(modulo, model_str):
                logger.error(f"Class not found {model_str} in {ruta_modulo}")
                continue
            load = False
            for row in reader:
                row = row | {"context": context}
                self.db_manager.create_document(db_name=None, document_class=getattr(modulo, model_str), **row)
                if not load:
                    load = True
            attr_data = {f"set__data__{idx}__load": load}
            self.db_manager.update(micro, **attr_data)
            if load:
                logger.info(f"Load data {micro.code} - {tenant} - {file.get('path')}")


class UserStub(object):
    def __init__(self):
        cloud_map = CloudMap(service_name=Config.SERVICE_NAME_BALANCER)
        response = cloud_map.discover_instances()
        self.port = None
        self.host = None
        for instance in response:
            if instance.get("InstanceId") == Config.SAAS_MS_USER:
                self.host = instance.get("Attributes").get("host")
                self.port = instance.get("Attributes").get("port")
                break

    def create_users(self, context: dict, list_value: list) -> user_pb2.UserCreateResponse:
        credentials = grpc.ssl_channel_credentials()
        with grpc.secure_channel(
            f"{self.host}:{self.port}",
            credentials,
            options=[("grpc.ssl_target_name_override", "omni.pro")],
        ) as channel:
            stub = user_pb2_grpc.UsersServiceStub(channel)
            response = user_pb2.UserCreateResponse()
            for value in list_value:
                context["user"] = context.get("user") or value.get("username")
                response = self._create_user(context, value, stub)
                if response.response_standard.status_code in range(200, 300):
                    logger.info(f"User created {response.user.username}")
                else:
                    logger.error(f"User not created {context.get('user')}")
            return response

    def _create_user(self, context: dict, value: dict, stub) -> user_pb2.UserCreateResponse:
        user = user_pb2.UserCreateRequest(
            **{
                "context": context,
                "email": value.get("email"),
                "email_confirm": value.get("email"),
                "language": {"code": "01", "code_name": "CO"},
                "name": value.get("name"),
                "password": value.get("password"),
                "password_confirm": value.get("password"),
                "timezone": {"code": "01", "code_name": "CO"},
                "username": value.get("username"),
                "is_superuser": wrappers_pb2.BoolValue(value=util.parse_bool(value.get("is_superuser") or False)),
            }
        )
        return stub.UserCreate(user)



from omni.pro.models.base import BaseModel
from omni.pro.models.sql.catalog.category import Category
from omni.pro.models.sql.rules.delivery_category import DeliveryCategory
from omni.pro.protos.v1.rules.delivery_category_category_pb2 import (
    DeliveryCategoryCategory as DeliveryCategoryCategoryProto,
)
from peewee import ForeignKeyField

# TODO tabla intermedia entre delivery_category y category


class DeliveryCategoryCategory(BaseModel):
    delivery_category_id = ForeignKeyField(DeliveryCategory, on_delete="RESTRICT")
    category_id = ForeignKeyField(Category, on_delete="RESTRICT")

    def to_proto(self):
        return DeliveryCategoryCategoryProto(
            id=self.id,
            delivery_category_id=self.delivery_category_id.id,
            category_id=self.category_id.id,
            active=self.active,
            object_audit=self.get_audit_proto(),
        )

    class Meta:
        table_name = "delivery_category_category"



from omni.pro.models.base import BaseModel
from omni.pro.models.sql.rules.delivery_method import DeliveryMethod
from omni.pro.models.stock.warehouse import Warehouse
from omni.pro.protos.v1.rules.delivery_warehouse_ref_pb2 import DeliveryWarehouseRef as DeliveryWarehouseRefProto
from peewee import DeferredThroughModel, ForeignKeyField


# TODO tabla intermedia entre delivery_method y warehouse
class DeliveryWarehouseRef(BaseModel):
    delivery_method_id = ForeignKeyField(DeliveryMethod, on_delete="RESTRICT")
    warehouse_id = ForeignKeyField(Warehouse, on_delete="RESTRICT")

    def to_proto(self):
        return DeliveryWarehouseRefProto(
            id=self.id,
            delivery_method_id=self.delivery_method_id.id,
            warehouse_id=self.warehouse_id.id,
            active=self.active,
            object_audit=self.get_audit_proto(),
        )

    class Meta:
        table_name = "delivery_warehouse_ref"


DeferredThroughModel.set_model(DeliveryMethod.delivery_warehouse_ids, DeliveryWarehouseRef)



from google.protobuf.timestamp_pb2 import Timestamp
from peewee import CharField, DateTimeField, DecimalField, ForeignKeyField

from omni.pro.models.base import BaseModel
from omni.pro.models.stock.attachment import Attachment
from omni.pro.models.stock.carrier import Carrier
from omni.pro.models.stock.location import Location
from omni.pro.models.stock.picking_type import PickingType
from omni.pro.models.stock.procurement_group import ProcurementGroup
from omni.pro.models.stock.user import User
from omni.pro.protos.v1.stock.picking_pb2 import Picking as PickingProto


class Picking(BaseModel):
    name = CharField()
    picking_type_id = ForeignKeyField(PickingType, on_delete="RESTRICT")
    location_id = ForeignKeyField(Location, on_delete="RESTRICT")
    location_dest_id = ForeignKeyField(Location, on_delete="RESTRICT")
    user_id = ForeignKeyField(User, on_delete="RESTRICT")
    attachment_guide_id = ForeignKeyField(Attachment, on_delete="RESTRICT")
    attachment_invoice_id = ForeignKeyField(Attachment, on_delete="RESTRICT")
    origin = CharField()
    date_done = DateTimeField(null=True, default=None)
    scheduled_date = DateTimeField()
    date_start_preparation = DateTimeField()
    time_total_preparation = DecimalField()
    time_assigned = DecimalField()
    carrier_id = ForeignKeyField(Carrier, on_delete="RESTRICT")
    date_delivery = DateTimeField(null=True, default=None)
    carrier_tracking_ref = CharField()
    group_id = ForeignKeyField(ProcurementGroup, on_delete="RESTRICT")
    weight = DecimalField()
    shipping_weight = DecimalField()

    class Meta:
        table_name = "picking"

    def to_proto(self):
        date_done = None
        if self.date_done:
            date_done = Timestamp()
            date_done.FromDatetime(self.date_done)
            self.date_done = date_done
        if self.date_start_preparation:
            self.date_start_preparation = Timestamp().FromDatetime(self.date_start_preparation)
        scheduled_date = Timestamp()
        scheduled_date.FromDatetime(self.scheduled_date)
        date_delivery = Timestamp()
        date_delivery.FromDatetime(self.date_delivery)

        return PickingProto(
            object_audit=self.get_audit_proto(),
            id=self.id,
            name=self.name,
            picking_type_id=self.picking_type_id.id,
            location_id=self.location_id.id,
            location_dest_id=self.location_dest_id.id,
            user_id=self.user_id.id,
            attachment_guide_id=self.attachment_guide_id.id,
            attachment_invoice_id=self.attachment_invoice_id.id,
            origin=self.origin,
            date_done=self.date_done,
            scheduled_date=scheduled_date,
            date_start_preparation=self.date_start_preparation,
            time_total_preparation=self.time_total_preparation,
            time_assigned=self.time_assigned,
            carrier_id=self.carrier_id.id,
            date_delivery=date_delivery,
            carrier_tracking_ref=self.carrier_tracking_ref,
            group_id=self.group_id.id,
            weight=self.weight,
            shipping_weight=self.shipping_weight,
            active=self.active,
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import delivery_method_warehouse_pb2 as v1_dot_rules_dot_delivery__method__warehouse__pb2


class DeliveryMethodWarehouseServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.DeliveryMethodWarehouseCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseCreate",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseCreateResponse.FromString,
        )
        self.DeliveryMethodWarehouseRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseRead",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseReadResponse.FromString,
        )
        self.DeliveryMethodWarehouseUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseUpdate",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseUpdateResponse.FromString,
        )
        self.DeliveryMethodWarehouseDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseDelete",
            request_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseDeleteResponse.FromString,
        )


class DeliveryMethodWarehouseServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def DeliveryMethodWarehouseCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeliveryMethodWarehouseRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeliveryMethodWarehouseUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeliveryMethodWarehouseDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_DeliveryMethodWarehouseServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "DeliveryMethodWarehouseCreate": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseCreate,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseCreateResponse.SerializeToString,
        ),
        "DeliveryMethodWarehouseRead": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseRead,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseReadResponse.SerializeToString,
        ),
        "DeliveryMethodWarehouseUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseUpdate,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseUpdateResponse.SerializeToString,
        ),
        "DeliveryMethodWarehouseDelete": grpc.unary_unary_rpc_method_handler(
            servicer.DeliveryMethodWarehouseDelete,
            request_deserializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class DeliveryMethodWarehouseService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def DeliveryMethodWarehouseCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseCreate",
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseCreateRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeliveryMethodWarehouseRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseRead",
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseReadRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeliveryMethodWarehouseUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseUpdate",
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseUpdateRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeliveryMethodWarehouseDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseService/DeliveryMethodWarehouseDelete",
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseDeleteRequest.SerializeToString,
            v1_dot_rules_dot_delivery__method__warehouse__pb2.DeliveryMethodWarehouseDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/delivery_method_warehouse.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.rules import warehouse_hierarchy_pb2 as v1_dot_rules_dot_warehouse__hierarchy__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n(v1/rules/delivery_method_warehouse.proto\x12\x33pro.omni.oms.api.v1.rules.delivery_method_warehouse\x1a\x11\x63ommon/base.proto\x1a"v1/rules/warehouse_hierarchy.proto"\xc9\x02\n\x17\x44\x65liveryMethodWarehouse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12`\n\x1bhierarchi_warehouse_sort_by\x18\x03 \x01(\x0e\x32;.pro.omni.oms.api.v1.rules.delivery_method_warehouse.SortBy\x12\x61\n\x16transfer_warehouse_ids\x18\x04 \x03(\x0b\x32\x41.pro.omni.oms.api.v1.rules.warehouse_hierarchy.WarehouseHierarchy\x12\x0e\n\x06\x61\x63tive\x18\x05 \x01(\x08\x12?\n\x0cobject_audit\x18\x06 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xee\x01\n$DeliveryMethodWarehouseCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1e\n\x16transfer_warehouse_ids\x18\x02 \x03(\t\x12`\n\x1bhierarchi_warehouse_sort_by\x18\x03 \x01(\x0e\x32;.pro.omni.oms.api.v1.rules.delivery_method_warehouse.SortBy\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe3\x01\n%DeliveryMethodWarehouseCreateResponse\x12o\n\x19\x64\x65livery_method_warehouse\x18\x01 \x01(\x0b\x32L.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouse\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x80\x03\n"DeliveryMethodWarehouseReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9d\x02\n#DeliveryMethodWarehouseReadResponse\x12p\n\x1a\x64\x65livery_method_warehouses\x18\x01 \x03(\x0b\x32L.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouse\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xcf\x01\n$DeliveryMethodWarehouseUpdateRequest\x12o\n\x19\x64\x65livery_method_warehouse\x18\x01 \x01(\x0b\x32L.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouse\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe3\x01\n%DeliveryMethodWarehouseUpdateResponse\x12o\n\x19\x64\x65livery_method_warehouse\x18\x01 \x01(\x0b\x32L.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouse\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"j\n$DeliveryMethodWarehouseDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"r\n%DeliveryMethodWarehouseDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard*\x1b\n\x06SortBy\x12\x07\n\x03\x41SC\x10\x00\x12\x08\n\x04\x44\x45SC\x10\x01\x32\x86\x07\n\x1e\x44\x65liveryMethodWarehouseService\x12\xd8\x01\n\x1d\x44\x65liveryMethodWarehouseCreate\x12Y.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseCreateRequest\x1aZ.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseCreateResponse"\x00\x12\xd2\x01\n\x1b\x44\x65liveryMethodWarehouseRead\x12W.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseReadRequest\x1aX.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseReadResponse"\x00\x12\xd8\x01\n\x1d\x44\x65liveryMethodWarehouseUpdate\x12Y.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseUpdateRequest\x1aZ.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseUpdateResponse"\x00\x12\xd8\x01\n\x1d\x44\x65liveryMethodWarehouseDelete\x12Y.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseDeleteRequest\x1aZ.pro.omni.oms.api.v1.rules.delivery_method_warehouse.DeliveryMethodWarehouseDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.delivery_method_warehouse_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_SORTBY"]._serialized_start = 2294
    _globals["_SORTBY"]._serialized_end = 2321
    _globals["_DELIVERYMETHODWAREHOUSE"]._serialized_start = 153
    _globals["_DELIVERYMETHODWAREHOUSE"]._serialized_end = 482
    _globals["_DELIVERYMETHODWAREHOUSECREATEREQUEST"]._serialized_start = 485
    _globals["_DELIVERYMETHODWAREHOUSECREATEREQUEST"]._serialized_end = 723
    _globals["_DELIVERYMETHODWAREHOUSECREATERESPONSE"]._serialized_start = 726
    _globals["_DELIVERYMETHODWAREHOUSECREATERESPONSE"]._serialized_end = 953
    _globals["_DELIVERYMETHODWAREHOUSEREADREQUEST"]._serialized_start = 956
    _globals["_DELIVERYMETHODWAREHOUSEREADREQUEST"]._serialized_end = 1340
    _globals["_DELIVERYMETHODWAREHOUSEREADRESPONSE"]._serialized_start = 1343
    _globals["_DELIVERYMETHODWAREHOUSEREADRESPONSE"]._serialized_end = 1628
    _globals["_DELIVERYMETHODWAREHOUSEUPDATEREQUEST"]._serialized_start = 1631
    _globals["_DELIVERYMETHODWAREHOUSEUPDATEREQUEST"]._serialized_end = 1838
    _globals["_DELIVERYMETHODWAREHOUSEUPDATERESPONSE"]._serialized_start = 1841
    _globals["_DELIVERYMETHODWAREHOUSEUPDATERESPONSE"]._serialized_end = 2068
    _globals["_DELIVERYMETHODWAREHOUSEDELETEREQUEST"]._serialized_start = 2070
    _globals["_DELIVERYMETHODWAREHOUSEDELETEREQUEST"]._serialized_end = 2176
    _globals["_DELIVERYMETHODWAREHOUSEDELETERESPONSE"]._serialized_start = 2178
    _globals["_DELIVERYMETHODWAREHOUSEDELETERESPONSE"]._serialized_end = 2292
    _globals["_DELIVERYMETHODWAREHOUSESERVICE"]._serialized_start = 2324
    _globals["_DELIVERYMETHODWAREHOUSESERVICE"]._serialized_end = 3226
# @@protoc_insertion_point(module_scope)
